import type { ColorTextureFormat, DepthStencilTextureFormat, TextureFormat } from "../types/texture-formats.js";
import type { Device } from "../device.js";
import { Resource, ResourceProps } from "./resource.js";
import { Texture } from "./texture.js";
import { TextureView } from "./texture-view.js";
export type FramebufferProps = ResourceProps & {
    width?: number;
    height?: number;
    colorAttachments?: (TextureView | Texture | ColorTextureFormat)[];
    depthStencilAttachment?: (TextureView | Texture | DepthStencilTextureFormat) | null;
};
/**
 * Create new textures with correct size for all attachments.
 * @note resize() destroys existing textures (if size has changed).
 */
export declare abstract class Framebuffer extends Resource<FramebufferProps> {
    static defaultProps: Required<FramebufferProps>;
    get [Symbol.toStringTag](): string;
    /** Width of all attachments in this framebuffer */
    width: number;
    /** Height of all attachments in this framebuffer */
    height: number;
    /** Color attachments */
    colorAttachments: TextureView[];
    /** Depth-stencil attachment, if provided */
    depthStencilAttachment: TextureView | null;
    constructor(device: Device, props?: FramebufferProps);
    /**
     * Resizes all attachments
     * @note resize() destroys existing textures (if size has changed).
     */
    resize(size: {
        width: number;
        height: number;
    }): void;
    resize(size: [width: number, height: number]): void;
    resize(): void;
    /** Auto creates any textures */
    protected autoCreateAttachmentTextures(): void;
    /** Create a color texture */
    protected createColorTexture(format: TextureFormat): Texture;
    /** Create depth stencil texture */
    protected createDepthStencilTexture(format: TextureFormat): Texture;
    /**
     * Default implementation of resize
     * Creates new textures with correct size for all attachments.
     * and destroys existing textures if owned
     */
    protected resizeAttachments(width: number, height: number): void;
}
//# sourceMappingURL=framebuffer.d.ts.map