import { ShaderPass } from "../../../lib/shader-module/shader-pass.js";
/**
 * Tilt Shift
 * Simulates the shallow depth of field normally encountered in close-up photography
 */
export type TiltShiftProps = {
    /** The x,y coordinate of the start of the line segment. */
    start?: [number, number];
    /** The xm y coordinate of the end of the line segment. */
    end?: [number, number];
    /** The maximum radius of the pyramid blur. */
    blurRadius?: number;
    /** The distance from the line at which the maximum blur radius is reached. */
    gradientRadius?: number;
    /** @deprecated internal shaderpass use */
    invert?: number;
};
/**
 * Tilt Shift
 * Simulates the shallow depth of field normally encountered in close-up
 * photography, which makes the scene seem much smaller than it actually
 * is. This filter assumes the scene is relatively planar, in which case
 * the part of the scene that is completely in focus can be described by
 * a line (the intersection of the focal plane and the scene). An example
 * of a planar scene might be looking at a road from above at a downward
 * angle. The image is then blurred with a blur radius that starts at zero
 * on the line and increases further from the line.
 */
export declare const tiltShift: ShaderPass<TiltShiftProps, TiltShiftProps>;
//# sourceMappingURL=tiltshift.d.ts.map